package cn.estsh.i3plus.core.api.iservice.busi;

import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.impp.framework.base.service.ICrudService;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * @Description : 系统角色业务接口
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-22 16:58:43.771
 * @Modify :
 **/
public interface ISysRoleService extends ICrudService<SysRole> {

    @ApiOperation(value = "更新角色信息", notes = "根据ID 批量更新角色状态信息")
    void updateSysRoleStatusByIds(Long[] ids, int status, SessionUser user);

    @ApiOperation(value = "刷新角色菜单关系", notes = "刷新 角色 菜单关系信息")
    void refreshSysRoleRef(Long roleId, Long[] ids, String userName);

    @ApiOperation(value = "刷新角色菜单关系", notes = "批量刷新 角色 菜单关系信息")
    void refreshBatchSysRoleRef(Long[] roleIds, Long[] ids, String userName);

    @ApiOperation(value = "根据角色ID更新角色菜单关系")
    void updateRoleMenuChangeRoleNameRdd(Long roleId, String roleName);

    void refreshSysRoleRef(Long roleId, Integer softType, List<Long> idList, String userName);

    @ApiOperation(value = "查信角色菜单关系", notes = "查询 ID 查询角色菜单关系")
    List<SysRefRoleMenu> findSysRefRoleMenuByRoleId(Long roleId);

    @ApiOperation(value = "批量查询角色所有菜单", notes = "批量查询角色所有菜单")
    List<SysRefRoleMenu> findSysRefRoleMenuByRoleIdList(List<Long> idList);

    @ApiOperation(value = "查信角色菜单关系", notes = "查询 ID 查询角色菜单关系")
    List<SysRefRoleMenu> findSysRefRoleMenuByRoleId(Long roleId, Integer softType);

    @ApiOperation(value = "查信角色信息", notes = "根据ID + 角色状态查询角色信息")
    List<SysRole> findSysRoleByInIdAndStatus(Long[] ids, Integer status);

    @ApiOperation(value = "查有效的角色信息", notes = "根据ID + 查有效的角色信息")
    List<SysRole> findValidSysRole();

    @ApiOperation(value = "查信用户的所有角色信息", notes = "查信用户的所有角色信息")
    List<SysRole> findSysRoleByUser(SysUser user);

    @ApiOperation(value = "查信角色信息", notes = "查询角色信息(分页/组合)")
    ListPager<SysRole> querySysRole(SysRole role, List<Long> idList, Pager pager);

    /***
     * 检查角色唯一性
     * @param role 角色信息
     */
    @ApiOperation(value = "角色唯一检查")
    void checkSysRoleOnly(SysRole role);

    /**
     * 根据sysRole 查询符合条件的角色
     *
     * @param sysRole
     * @return
     */
    @ApiOperation(value = "查询符合条件的角色")
    List<SysRole> findAllByBaseBean(SysRole sysRole);

    /**
     * 根据sysRole属性 查询符合条件的角色
     *
     * @param sysRole
     * @return
     */
    @ApiOperation(value = "查询符合条件的角色")
    List<SysRole> findAllByRoleProperty(SysRole sysRole);

    /**
     * 查询所有的角色名称+id
     * @return
     */
    List<SysRole> findAllCommonRoles();

    /**
     * 根据sysRole属性 新增用户角色变更日志
     *
     * @param sysRole
     * @return
     */
    @ApiOperation(value = "新增用户角色变更日志")
    SysLogRoleOperate insertSysLogRoleOperate(SysRole sysRole, Integer status);

    /**
     * 根据sysRole属性 新增用户角色变更日志通过描述
     *
     * @param sysRole
     * @return
     */
    @ApiOperation(value = "新增用户角色变更日志通过描述")
    SysLogRoleOperate insertSysLogRoleOperate(SysRole sysRole, String statusDecription);

    /**
     * sysLogRoleMenuChange 新增角色菜单权限变更日志
     *
     * @param sysLogRoleMenuChange
     * @return
     */
    @ApiOperation(value = "新增角色菜单权限变更日志")
    SysLogRoleMenuChange insertSysLogRoleMenuChange(SysLogRoleMenuChange sysLogRoleMenuChange);

    List<SysRefRoleMenu> setLanguageSysMenuName(String languageCode, List<SysRefRoleMenu> menuList);
}
