package cn.estsh.i3plus.core.api.iservice.busi;

import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.shirotoken.BaseToken;
import cn.estsh.i3plus.pojo.model.platform.AndonQueryUserModel;
import cn.estsh.i3plus.pojo.model.platform.AndonQueryUserResultModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authc.AuthenticationException;

import java.util.List;

/**
 * @Description : 账号账号服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-25 16:21
 * @Modify :
 **/
public interface ISysUserService {

    /**
     * 账号登录
     *
     * @param loginName    登录名
     * @param password     登录密码
     * @param languageCode 界面语言
     * @return
     */
    @ApiOperation(value = "账号登录", notes = "普通账号登录功能")
    ResultBean<SessionUser> loginUser(String loginName, String password, String languageCode, Integer loginPlatform, String deviceId) throws AuthenticationException;

    @ApiOperation(value = "获取Token", notes = "根据登录名判断是否是管理员")
    BaseToken getToken(String loginName, String password, String languageCode, Integer loginPlatform);

    /**
     * 修改状态
     *
     * @param id     修改账号状态
     * @param status 状态
     * @param user   当前会话账号
     * @return
     */
    @ApiOperation(value = "修改状态", notes = "修改指定账号状态")
    void updateSysUserStatus(Long id, int status, SessionUser user);

    @ApiOperation(value = "锁定用户", notes = "锁定用户")
    void doLockSysUserById(Long id);

    @ApiOperation(value = "锁定用户指定状态", notes = "锁定用户指定状态")
    void doLockAndStatusSysUserById(Long id, Integer status);

    /**
     * 批量修改账号状态
     *
     * @param ids
     * @param status
     * @param user
     */
    @ApiOperation(value = "修改状态", notes = "批量修改指定账号状态")
    void updateBatchSysUserStatus(Long[] ids, int status, SessionUser user);

    /**
     * 根据用户名批量修改账号状态
     *
     * @param userLoginNameList
     * @param status
     */
    @ApiOperation(value = "根据用户名批量修改账号状态", notes = "根据用户名批量修改账号状态")
    void updateBatchSysUserStatusByUserLoginNames(List<String> userLoginNameList, int status);

    /**
     * 修改账号信息
     *
     * @param sysUser
     */
    @ApiOperation(value = "修改账号信息", notes = "修改账号信息")
    void updateSysUser(SysUser sysUser);

    /**
     * 修改账号密码
     *
     * @param userId   需要修改的账号ID
     * @param password 旧密码
     * @param newPwd   新密码
     */
    @ApiOperation(value = "修改号密码", notes = "修改账号的密码信息")
    void updateSysUserPassword(Long userId, String password, String newPwd) throws Exception;

    /**
     * 根据登录名修改密码，供应商已经通过验证码验证
     * @param loginName
     * @param password
     */
    @ApiOperation(value = "供应商修改密码",notes = "供应商修改密码")
    void updateSysUserPwdByLoginName(String loginName,String password);

    /**
     * 刷新账号登录信息
     *
     * @param userId
     */
    @ApiOperation(value = "刷新账号", notes = "刷新账号登录信息")
    void refreshUserLoginInformation(Long userId);

    /**
     * 首次登录重置账号登录次数
     *
     * @param userId
     */
    @ApiOperation(value = "首次登录重置账号登录次数", notes = "首次登录重置账号登录次数")
    void updateUserLoginNum(Long userId,Integer userLoginNum);

    /**
     * 手动重置密码后是否强制修改密码
     *
     * @param userId
     */
    @ApiOperation(value = "手动重置密码后是否强制修改密码", notes = "手动重置密码后是否强制修改密码")
    void updateUserLoginNumByHeadResetPwd(Long userId,Integer userLoginNum);

    /**
     * 首次登录重置账号登录次数
     *
     * @param userId
     */
    @ApiOperation(value = "重置账号修改密码频次", notes = "重置账号修改密码频次")
    void updateUserModifyPwdNum(Long userId,Integer modifyPwdNum);


    /**
     * 刷新账号登录时间信息
     *
     * @param userId
     */
    @ApiOperation(value = "刷新账号登录时间信息", notes = "刷新账号登录时间信息")
    void refreshUserLoginDateTime(Long userId,Long userInfoId);

    /**
     * 新增账号信息
     *
     * @param sysUser 新增账号
     * @return
     */
    @ApiOperation(value = "新增账号", notes = "新增账号信息")
    SysUser insertSysUser(SysUser sysUser) throws Exception;

    /**
     * 添加 账号登录信息
     *
     * @param sysLogUserLogin
     * @return
     */
    @ApiOperation(value = "新增登录日志", notes = "新增登录日志信息")
    SysLogUserLogin insertSysLogUserLogin(SysLogUserLogin sysLogUserLogin);

    /**
     * 添加 账号状态变更日志
     *
     * @param sysUser
     * @return
     */
    @ApiOperation(value = "新增账号状态变更日志", notes = "新增账号状态变更日志")
    SysLogUserStatus insertSysLogUserStatus(SysUser sysUser, Integer status);

    /**
     * 查询用户登录日志
     *
     * @param startTime 开始日期
     * @param endTime   结束日期
     * @return
     */
    @ApiOperation(value = "查询用户登录日志")
    List<SysLogUserLogin> queryUserMonthlyLoginLog(String startTime, String endTime);

    /**
     * 查询用户登录失败日志
     *
     * @param startTime 开始日期
     * @param endTime   结束日期
     * @return
     */
    @ApiOperation(value = "查询用户登录失败日志")
    List<SysLogUserLogin> queryUserMonthlyFailedLoginLog(String startTime, String endTime);

    /**
     * 刷新账号角色关系
     *
     * @param sysUser
     * @param roleIds
     * @param userName
     */
    @ApiOperation(value = "刷新账号", notes = "刷新账号角色关系")
    void refreshSysRefUserRole(SysUser sysUser, Long[] roleIds, String userName);

    /**
     * 根据Key 删除数据
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "删除账号", notes = "根据ID 删除张哈信息")
    void deleteSysUserById(Long id);

    /**
     * 批量删除账号 逻辑删除
     *
     * @param ids  需要删除的账号ID
     * @param user 当前会话账号
     */
    @ApiOperation(value = "删除账号", notes = "根据ID 批量删除张哈信息")
    void deleteBatchSysUserById(Long[] ids, SessionUser user);

    /**
     * 查询所有数据
     *
     * @return
     */
    @ApiOperation(value = "查询所有", notes = "查询所有账号信息")
    List<SysUser> list();

    /**
     * 查询所有数据
     *
     * @return
     */
    @ApiOperation(value = "查询所有", notes = "查询所有账号信息")
    List<SysUser> listByUserOrganizeCode();

    /**
     * 查询所有数据
     *
     * @return
     */
    @ApiOperation(value = "查询所有用户和角色信息", notes = "查询所有账号信息")
    List<SysUser> listWithRole();

    /**
     * 通过roleId查询所有用户和角色信息
     *
     * @return
     */
    @ApiOperation(value = "查询所有用户和角色信息", notes = "查询所有账号信息")
    List<SysUser> listWithRoleByRoleId(Long roleId);

    /**
     * 查询所有数据
     *
     * @return
     */
    @ApiOperation(value = "获取用户状态描述", notes = "获取用户状态描述")
    List<SysUser> findSysUserStatusRdd(List<SysUser> userList);

    /**
     * 根据角色查询用户信息
     *
     * @param roleId 角色ID
     * @return 获取用户ID 集合
     */
    @ApiOperation(value = "查询用户ID 集合", notes = "根据角色Id 查询用户ID集合")
    List<Long> findSysUserIdListByRoleId(Long roleId);

    /***
     * 根据 部门ID 获取用户ID
     * @param idList 部门集合
     * @return 用户ID集合
     */
    @ApiOperation(value = "查询用户ID 集合", notes = "根据部门Id 查询用户ID集合")
    List<Long> findSysUserIdListByDepartmentId(List<Long> idList);

    /**
     * 复杂查询
     *
     * @param user
     * @param pager
     * @return
     */
    @ApiOperation(value = "账号查询", notes = "查询账号信息-复杂查询")
    ListPager<SysUser> querySysUserByPager(SysUser user, Pager pager);

    @ApiOperation(value = "账号查询", notes = "查询账号信息-复杂查询")
    ListPager<SysUser> querySysUserByPager(SysUser user, List<Long> idList, Pager pager);


    List<SysUser> querySysUserByOrganizeCode(String orgCode);

    /**
     * 根据条件查询账号
     * @param sysUser  账户条件对象
     * @return 账户数组
     */
    List<SysUser> querySysUser(SysUser sysUser);

    /**
     * 根据id 数据
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "账号查询", notes = "查询账号信息-根据ID查询")
    SysUser getSysUserById(Long id);

    /**
     * 获取账号详细信息
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "获取用户信息", notes = "查询用户信息-根据ID查询")
    SysUserInfo getSysUserInfoById(Long id);

    /**
     * 根据登录名称 获取账号信息
     *
     * @param loginName
     * @return
     */
    @ApiOperation(value = "获取账号信息", notes = "查询用户信息-根据登录名称查询")
    SysUser getSysUserByLoginName(String loginName);

    /**
     * 根据登录名+email获取账号信息
     * @param loginName
     * @param email
     * @return
     */
    @ApiOperation(value = "获取账号信息",notes = "查询用户信息-根据登录名称+邮箱查询")
    SysUser getSysUserByLoginNameAndEmail(String loginName,String email);

    /**
     * 根据手机号查询账号
     *
     * @param email
     * @return
     */
    @ApiOperation(value = "获取账号信息", notes = "查询用户信息-根据邮箱查询")
    SysUser getSysUserByEmail(String email);

    /**
     * 根据手机号查询账号
     *
     * @param phone
     * @return
     */
    @ApiOperation(value = "获取账号信息", notes = "查询用户信息-根据手机查询")
    SysUser getSysUserByPhone(String phone);

    /**
     * 查出账号角色关系
     *
     * @param userId
     * @return
     */
    @ApiOperation(value = "获取用户角色信息", notes = "获取指定账号的所有角色信息")
    List<SysRefUserRole> findSysRefUserRoleByUserId(Long userId);

    /**
     * 查出账号角色关系
     *
     * @param userId
     * @return
     */
    @ApiOperation(value = "查询当前用户角色下属所有菜单", notes = "查询当前用户角色下属所有菜单")
    List<SysRefRoleMenu> findSysRefUserRoleMenuByUserId(Long userId);

    /**
     * 判断账号名是否存在
     *
     * @param userLoginName
     * @param id
     * @return
     */
    @ApiOperation(value = "账号检查", notes = "检查账号的登录名是否重复")
    boolean checkSysUserLoginName(String userLoginName, long id);

    /**
     * 判断邮箱是否存在
     *
     * @param email
     * @param id
     * @return
     */
    @ApiOperation(value = "账号检查", notes = "检查账号的邮箱是否重复")
    boolean checkSysUserEmail(String email, long id);

    /**
     * 判断手机是否存在
     *
     * @param phone
     * @param id
     * @return
     */
    @ApiOperation(value = "账号检查", notes = "检查账号的手机号是否重复")
    boolean checkSysUserPhone(String phone, long id);

    /**
     * 账号唯一教研
     *
     * @param user
     */
    @ApiOperation(value = "账号检查", notes = "检查账号信息唯一(登录名、邮箱、手机号)")
    void checkSysUserOnly(SysUser user);

    /**
     * 获取用户数量
     *
     * @return 用户数量
     */
    @ApiOperation(value = "查询账户数量")
    long getSysUserCount();

    /**
     * 登录封装账号信息
     *
     * @param sessionUser
     * @param user
     * @param userType
     * @param languageCode
     * @return
     */
    @ApiOperation(value = "封装用户信息", notes = "封装登录完成后的用户信息(角色、权限、部门、组织、岗位)")
    SessionUser packSessionUser(SessionUser sessionUser, SysUser user, Integer userType, Integer loginPlatform, String languageCode, String deviceId);

    @ApiOperation(value = "封装用户信息", notes = "封装用户列表分页信息")
    SessionUser packPageSessionUser(SessionUser sessionUser, SysUser user);

    @ApiOperation(value = "封装用户信息", notes = "封装用户系统配置信息")
    SessionUser packConfigSessionUser(SessionUser sessionUser, SysUser user);

    /**
     * 根据id查询用户信息
     *
     * @param ids 用户id
     * @return 查询结果
     */
    @ApiOperation(value = "根据id查询用户信息")
    List<SysUser> findSysUserByIds(Long[] ids);


    /**
     * 根据用户名查询密码是否正确
     *
     * @param loginName 用户名
     * @param password  密码
     */
    @ApiOperation(value = "根据用户名查询密码是否正确")
    SysUser validatePasswordByLoginNameAndReturnUser(String loginName, String password);

    @ApiOperation(value = "查询有效的用户与角色关系")
    List<SysRefUserRole> findValidUserRoleRef();

    @ApiOperation(value = "查询所有用户")
    List<SysUser> findAll();

    /**
     * 获取用户菜单
     * @param user
     */
    @ApiOperation(value = "获取用户菜单信息")
    void packUser(SysUser user);


    /**
     * 根据andon查询model查询用户信息
     * @param andonQueryUserModel
     * @return
     */
    @ApiOperation(value = "根据andon查询model查询用户信息")
    ListPager<AndonQueryUserResultModel> findUserByAndonQueryUserModel(AndonQueryUserModel andonQueryUserModel);

    /**
     * 转换sysUser to AndonQueryUserResultModel
     * @param sysUserList
     * @return
     */
     List<AndonQueryUserResultModel> packAndonQueryUserResultModel(List<SysUser> sysUserList);

    /**
     * 根据用户名列表批量查询用户
     * @param userLoginNamegenList
     * @return
     */
    @ApiOperation(value = "获取批量账号信息", notes = "查询用户信息-批量根据登录名称查询")
    List<SysUser> findSysUserByLoginName(List<String> userLoginNamegenList);


    /**
     * 根据用户名或者登录名分页查询
     * @param userLoginName
     * @param userName
     * @param pager
     * @return
     */
    @ApiOperation(value = "获取批量账号信息", notes = "查询用户信息-批量根据登录名称查询")
    ListPager<SysUser> findSysUserByName(String userLoginName,String userName ,Pager pager);

    /**
     * 查询长时间未登录账号
     * @return
     */
    @ApiOperation(value = "获取批量长时间未登录账号信息", notes = "查询用户信息-批量根据登录名称查询")
    List<SysUser> findUserNotLoginLongTime(String startDate);
}
