package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.busi.ICoreMemTreeService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysMenuService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysRoleService;
import cn.estsh.i3plus.core.apiservice.serviceimpl.busi.SysLocaleLanguageService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleLanguage;
import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
import cn.estsh.i3plus.pojo.platform.bean.SysRole;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.util.LocaleUtils;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import static cn.estsh.i3plus.platform.common.util.CommonConstWords.RESOURCE_PREFIX_MENU;


/**
 * @Description : 用户权限服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-25 21:01
 * @Modify :
 **/
@RestController
@Api(tags = "用户所有权限操作服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/auth")
public class AuthController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(AuthController.class);

    @Autowired
    private SysLocaleLanguageService localeLanguageService;

    @Autowired
    private ICoreMemTreeService memTreeService;

    @Autowired
    private ISysMenuService sysMenuService;

    @Autowired
    private ISysRoleService sysRoleService;

    /**
     * 获取登录用户信息
     *
     * @return 处理结果
     */
    @GetMapping(value = "/get-user-info")
    @ApiOperation(value = "获取登录用户信息", notes = "获取当前登录用户详细信息")
    public ResultBean getUserInfo() {
        LOGGER.info("获取用户信息...lang:{}", AuthUtil.getSessionAttribute("languageCode"));
        try {
            SessionUser sUser = AuthUtil.getSessionUser();
            LOGGER.info("用户：{}", sUser);
            return new ResultBean(true, "", sUser.getUserInfo());
        } catch (ImppBusiException e) {
            return new ResultBean(false, e.getErrorDetail() + "-[解决]" + e.getErrorSolution());
        }
    }

    /**
     * 获取登录用户信息
     *
     * @return 处理结果
     */
    @GetMapping(value = "/get-user")
    @ApiOperation(value = "获取登录用户信息", notes = "获取当前登录用户账号详细信息")
    public ResultBean getUser() {
        LOGGER.info("获取用户信息...lang:{}", AuthUtil.getSessionAttribute("languageCode"));
        try {
            SessionUser sUser = AuthUtil.getSessionUser();
            LOGGER.info("用户：{}", sUser);
            return new ResultBean(true, "", sUser.getUser());
        } catch (ImppBusiException e) {
            return new ResultBean(false, e.getErrorDetail() + "-[解决]" + e.getErrorSolution());
        }
    }

    @GetMapping(value = "/get-user-type")
    @ApiOperation(value = "获取登录用户信息", notes = "获取当前登录用户账号详细信息")
    public ResultBean getUserType() {
        LOGGER.info("获取用户信息...lang:{}", AuthUtil.getSessionAttribute("languageCode"));
        try {
            CommonEnumUtil.USER_TYPE type = AuthUtil.getUserType();
            return new ResultBean(true, "", type);
        } catch (ImppBusiException e) {
            return new ResultBean(false, e.getErrorDetail() + "-[解决]" + e.getErrorSolution());
        }
    }


    /**
     * 获取登录用户信息
     *
     * @return 处理结果
     */
    @GetMapping(value = "/get-user-organize")
    @ApiOperation(value = "获取会话用户组织信息", notes = "获取当前登录用户账号详细信息")
    public ResultBean getUserOrganize() {
        LOGGER.info("获取用户信息...lang:{}", AuthUtil.getSessionAttribute("languageCode"));
        try {
            return new ResultBean(true, "", AuthUtil.getOrganize());
        } catch (ImppBusiException e) {
            return new ResultBean(false, e.getErrorDetail() + "-[解决]" + e.getErrorSolution());
        }
    }

    /**
     * 获取登录用户功能权限信息
     *
     * @return 处理结果
     */
    @GetMapping(value = "/menu/find-home-list")
    @ApiOperation(value = "获取登录用户功能权限信息", notes = "获取首页权限信息")
    public ResultBean findHomeList() {
        Long startTime = System.currentTimeMillis();
        List<SysMenu> menuList = memTreeService.packTreeSysMenuByUserId(getSessionUser().getUser().getId());
        Long menuSelectTime = System.currentTimeMillis();
        LOGGER.info("第一段耗时：{}", menuSelectTime - startTime);
        List<SysMenu> list = memTreeService.packTreeSysMenuByParentIdAndMenuType(menuList, CommonConstWords.SYSTEM_MENU_ROOT_ID, CommonEnumUtil.METHOD_LEVEL.PLUGIN.getValue());
        LOGGER.info("第二段耗时：{}", System.currentTimeMillis() - menuSelectTime);

        if (list != null && list.size() > 0) {
            // 翻译第一级
            list.forEach(menu -> menu.setName(LocaleUtils.getLocaleRes(RESOURCE_PREFIX_MENU + menu.getName())));
            // 内存排序  Sort 降序 code 升序
            list.sort(Comparator.comparing(SysMenu::getMenuSort).reversed().thenComparing(SysMenu::getMenuCode));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
        } else {
            return ResultBean.fail("操作失败").setCode(ResourceEnumUtil.MESSAGE.FAIL.getCode()).setErrorMsg("菜单信息不存在");
        }
    }

    /**
     * 获取登录用户功能权限信息
     *
     * @param parentId 菜单id
     * @return 处理结果
     */
    @GetMapping(value = "/menu/find-module-list/{parentId}")
    @ApiOperation(value = "获取登录用户功能权限信息", notes = "获取首页权限信息")
    public ResultBean findMenuModuleList(@PathVariable("parentId") String parentId, Integer menuSort) {
        try {
            ValidatorBean.checkIsNumber(parentId, "父节点不能为空");

            List<SysMenu> result = memTreeService.packTreeSysMenuByUserIdAndParentId(getSessionUser().getUser().getId(), Long.parseLong(parentId));
            //如果menuSort为0,则不用通过menuSort字段进行排序
            if (menuSort == null || menuSort != 0) {
                for (SysMenu sysMenu : result) {
                    sysMenu.setChildList(sysMenu.getChildList().stream().sorted(Comparator.comparing(SysMenu::getMenuSort)).collect(Collectors.toList()));
                }
                result = result.stream().sorted(Comparator.comparing(SysMenu::getMenuSort)).collect(Collectors.toList());
            }
            result = sysMenuService.setLanguageSysMenuName(AuthUtil.getSessionUser().getLanguageCode(), result);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(result);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            LOGGER.error(LocaleUtils.getEnumLocaleResDesc(ImppExceptionEnum.SYSTEM_EXCEPTION,
                    ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription()) + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    /**
     * 获取登录用户功能权限信息
     *
     * @param parentId 菜单id
     * @return 处理结果
     */
    @GetMapping(value = "/menu/list/{parentId}")
    @ApiOperation(value = "获取登录用户功能权限信息", notes = "获取首页权限信息")
    public ResultBean findModuleList(@PathVariable("parentId") String parentId) {
        try {
            ValidatorBean.checkIsNumber(parentId, "父节点不能为空");

            List<SysMenu> result = memTreeService.packTreeSysMenuByUserIdAndParentId(getSessionUser().getUser().getId(), Long.parseLong(parentId));
            // 国际化
            result = sysMenuService.setLanguageSysMenuName(getSessionUser().getLanguageCode(), result);
            result = findChildList(result);

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(result);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            LOGGER.error(LocaleUtils.getEnumLocaleResDesc(ImppExceptionEnum.SYSTEM_EXCEPTION,
                    ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription()) + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }


    private List<SysMenu> findChildList(List<SysMenu> findList) {
        List<SysMenu> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(findList)) {
            for (SysMenu menu : findList) {
                if (CollectionUtils.isNotEmpty(menu.getChildList())) {
                    result.addAll(findChildList(menu.getChildList()));
                }
                result.add(menu);
            }
        }
        return result;
    }


    /**
     * 修改用户会话语言
     *
     * @param code 语言代码
     * @return 处理结果
     */
    @PutMapping(value = "/update-language")
    @ApiOperation(value = "修改语言", notes = "修改用户会话语言")
    public ResultBean updateLanguage(String code) {
        try {
            SysLocaleLanguage language = localeLanguageService.getSysLocaleLanguageByCode(code);
            if (language != null) {
                AuthUtil.setSessionLanguage(language.getLanguageCode());
                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            } else {
                return ResultBean.fail("操作失败").setCode(ResourceEnumUtil.MESSAGE.FAIL.getCode()).setErrorMsg("语言信息不存在");
            }
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    /************************************ 权限分配是使用接口 ******************************************/

    /**
     * 获取登录用户功能权限信息
     *
     * @return 处理结果
     */
    @GetMapping(value = "/menu/module-list")
    @ApiOperation(value = "获取登录用户功能权限信息", notes = "获取首页权限信息")
    public ResultBean findModuleList() {
        List<SysMenu> list = null;
        if (AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)) {
            SysMenu sysMenu = new SysMenu();
            sysMenu.setMenuType(CommonEnumUtil.METHOD_LEVEL.PLUGIN.getValue());
            list = sysMenuService.findAllByBean(sysMenu).stream()
                    .filter(menu -> menu.getId() != 100000000L)
                    .collect(Collectors.toList());
        } else {
            List<SysMenu> menuList = memTreeService.packTreeSysMenuByUserId(getSessionUser().getUser().getId());
            list = memTreeService.packTreeSysMenuByParentIdAndMenuType(menuList, CommonConstWords.SYSTEM_MENU_ROOT_ID, CommonEnumUtil.METHOD_LEVEL.PLUGIN.getValue());


        }
        list = sysMenuService.setLanguageSysMenuName(AuthUtil.getSessionUser().getLanguageCode(), list);

        if (list != null && list.size() > 0) {
            // 内存排序  Sort 降序 code 升序
            list.sort(Comparator.comparing(SysMenu::getMenuSort).reversed().thenComparing(SysMenu::getMenuCode));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
        } else {
            return ResultBean.fail("操作失败").setCode(ResourceEnumUtil.MESSAGE.FAIL.getCode()).setErrorMsg("菜单信息不存在");
        }
    }
    /*------------------------数据版本记录功能查询------------------------------*/

    @GetMapping("/role/query")
    @ApiOperation(value = "获取所有的角色", notes = "获取所有的角色")
    public ResultBean getAllRoles() {
        List<SysRole> allCommonRoles;
        try {
            allCommonRoles = sysRoleService.findAllCommonRoles();
        } catch (Exception e) {
            return ResultBean.fail().setErrorMsg(e.getMessage());
        }
        return ResultBean.success().setResultList(allCommonRoles);
    }
//    /**
//     * 根据递归获取所有父节点 ID 获取所有 菜单
//     * @param list
//     * @param parentId
//     * @return
//     */
//    private List<SysMenu> findSysMenuChildList(List<SysMenu> list,Long parentId){
//        List<SysMenu> result = new ArrayList<>();
//        if(list != null && list.size() > 0){
//            for (SysMenu menu : list) {
//                if(menu.getParentId().equals(parentId)){
//                    result.add(menu);
//                }
//
//                List<SysMenu> childList = findSysMenuChildList(menu.getChildList(), parentId);
//                result.addAll( childList);
//            }
//        }
//        return result;
//    }

}
