package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.IPersonnelService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysLogRoleMenuChangeService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysLogRoleOperateService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysRoleService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.FileContentTypeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseBean;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.common.ExportDataModel;
import cn.estsh.i3plus.pojo.model.platform.SysRoleModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.i3plus.pojo.platform.repository.SysLogRoleOperateRepository;
import cn.estsh.impp.framework.base.controller.CrudBaseController;
import cn.estsh.impp.framework.base.service.ICrudService;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.management.relation.Role;
import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description : 系统角色对外接口
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-22 16:58:43.790
 * @Modify :
 **/
@RestController
@Api(tags = "系统角色服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/sys-role")
public class SysRoleController extends CrudBaseController<SysRole> {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysRoleController.class);

    @Autowired
    private ISysRoleService sysRoleService;

    @Autowired
    private IPersonnelService personnelService;

    @Autowired
    private SysLogRoleOperateRepository sysLogRoleOperateRDao;

    @Autowired
    private ISysLogRoleOperateService sysLogRoleOperateService;

    @Autowired
    private ISysLogRoleMenuChangeService sysLogRoleMenuChangeService;

    @Autowired
    private EntityManager entityManager;

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    @PostMapping({"/insert"})
    @ApiOperation(
            value = "新增数据",
            notes = "新增数据"
    )
    public ResultBean insert(SysRole sysRole) {
        try {
            sysRole = this.validatorInsertBean(sysRole);

            //新增用户角色操作变更日志
            sysRoleService.insertSysLogRoleOperate(sysRole, CommonEnumUtil.DAO_OPERATE_TYPE.INSERT.getValue());
            return ResultBean.success("操作成功").setResultObject(this.getCrudService().insert(sysRole)).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException var3) {
            return ResultBean.fail(var3);
        } catch (Exception var4) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(var4);
        }
    }

    @PutMapping({"/update"})
    @ApiOperation(
            value = "修改数据",
            notes = "根据ID 修改数据"
    )
    public ResultBean update(SysRole sysRole) {
        try {
            SysRole BeforeSysRole = sysRoleService.get(sysRole.getId());
            sysRole = this.validatorUpdateBean(sysRole);
            //修改角色菜单关联表中的角色名称
            sysRoleService.updateRoleMenuChangeRoleNameRdd(sysRole.getId(), sysRole.getName());
            //新增用户角色操作变更日志
            if (!BeforeSysRole.getName().equals(sysRole.getName())) {
                SysLogRoleOperate sysLogRoleOperate = new SysLogRoleOperate();
                sysLogRoleOperate.setRoleId(sysRole.getId());
                sysLogRoleOperate.setOperate(CommonEnumUtil.DAO_OPERATE_TYPE.UPDATE.getDescription());
                StringBuilder context = new StringBuilder();
                context.append("修改角色名称:" + BeforeSysRole.getName() + "->" + sysRole.getName());
                sysLogRoleOperate.setOperateContext(context.toString());
                sysLogRoleOperateRDao.save(sysLogRoleOperate);
            }
            return ResultBean.success("操作成功").setResultObject(this.getCrudService().update(sysRole)).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException var3) {
            return ResultBean.fail(var3);
        } catch (Exception var4) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(var4);
        }
    }

    @Override
    public ICrudService getCrudService() {
        return sysRoleService;
    }

    @Override
    public SysRole validatorInsertBean(SysRole role) throws Exception {
        //条件验证
        ValidatorBean.beginValid(role)
                .notNull("name", role.getName());

        sysRoleService.checkSysRoleOnly(role);

        //新增初始化
        ConvertBean.modelInitialize(role, AuthUtil.getSessionUser());
        role.setRoleStatus(CommonEnumUtil.DATA_STATUS.ENABLE.getValue());
        role.setMenuNumber(0);
        role.setModuleNumber(0);
        return role;
    }

    @Override
    public SysRole validatorUpdateBean(SysRole role) throws Exception {
        //条件验证
        ValidatorBean.beginValid(role)
                .notNull("id", role.getId())
                .notNull("name", role.getName());

        sysRoleService.checkSysRoleOnly(role);

        //新增初始化
        ConvertBean.modelUpdate(role, AuthUtil.getSessionUser());
        return role;
    }

    @DeleteMapping({"/delete/{id}"})
    @ApiOperation(value = "删除数据",notes = "根据ID删除数据")
    @Override
    public ResultBean delete(@PathVariable("id") Long id) {
        try {
            ValidatorBean.checkNotNull(id, "id不能为空");
            SysRole sysRole = sysRoleService.get(id);
            this.getCrudService().delete(new Long[]{id});
            //记录用户角色变更日志
            sysRoleService.insertSysLogRoleOperate(sysRole, CommonEnumUtil.DAO_OPERATE_TYPE.DELETE.getValue());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException var3) {
            return ResultBean.fail(var3);
        } catch (Exception var4) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(var4);
        }
    }

    @GetMapping(value = "/list-user")
    @ApiOperation(value = "查询角色", notes = "查询用户所拥有的角色信息")
    public ResultBean findSysRoleByUser() {
        try {
            if (!AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)) {
                List<SysRole> result = personnelService.findSysRoleByUserId(getSessionUser().getUser().getId());
                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                        .setResultList(result);
            }

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(sysRoleService.findAll());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询所有角色
     *
     * @param ids 角色id数组
     * @return 处理结果
     */
    @GetMapping(value = "/find-id-list")
    @ApiOperation(value = "查询所有角色", notes = "查询所有系统角色")
    public ResultBean findIdList(String[] ids) {
        try {
            List<SysRole> list = sysRoleService.findAll();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询角色的所有角色权限关系
     *
     * @param roleId 角色id
     * @return 处理结果
     */
    @GetMapping(value = "/find-ref-menu/{roleId}")
    @ApiOperation(value = "查询角色的所有角色权限关系", notes = "查询角色的所有角色权限关系")
    public ResultBean findRefRoleMenu(@PathVariable("roleId") Long roleId) {
        try {
            ValidatorBean.checkNotNull(roleId, "角色id 不能为空");

            List<SysRefRoleMenu> list = sysRoleService.findSysRefRoleMenuByRoleId(roleId);
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/export/ref-role-menu")
    @ApiOperation(value = "导出角色菜单关系", notes = "导出角色菜单关系")
    public ResultBean exportRoleMenu(HttpServletResponse response) {
        try {
            List<Long> roleIds = sysRoleService.findValidSysRole().stream().map(BaseBean::getId).collect(Collectors.toList());
            List<SysRefRoleMenu> list = new ArrayList<>();
            for (Long roleId : roleIds) {
                list.addAll(sysRoleService.findSysRefRoleMenuByRoleId(roleId));
            }

            if (CollectionUtils.isEmpty(list)) {
                return ResultBean.success("系统中不存在角色与菜单的关系。");
            }
            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            String fileName = "role-menu-ref-" + System.currentTimeMillis() + ".xls";
            File file = new File(fileName);
            file.createNewFile();
            File excel = excelTool.exportData(file, list, SysRefRoleMenu.class
                    , (LinkedHashMap<String, String>) ExcelTool.getColName(SysRefRoleMenu.class));

            response.setContentType("application/force-download"); // 设置强制下载不打开
            response.addHeader("Content-Disposition", "attachment;fileName=" + fileName); // 设置文件名
            response.addHeader("Content-type", FileContentTypeTool.getContentType(StringTool.getStringFileSuffix(fileName, true)));

            // 设置文件名
            try (BufferedInputStream bis = new BufferedInputStream(new DataInputStream(new FileInputStream(excel)))) {
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                LOGGER.error("用户角色关联导出异常", e);
            }

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @Override
    @GetMapping(value = "/query")
    @ApiOperation(value = "查询角色", notes = "组合查询角色，外带分页功能")
    public ResultBean queryPager(SysRole bean, Pager pager) {
        try {
            ListPager result = null;
            if (AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)) {
                result = sysRoleService.findPager(bean, pager);
            } else {
                List<SysRefUserRole> refList = personnelService.findSysRefUserRole(getSessionUser().getUser().getId());
                if (refList != null && refList.size() > 0) {
                    List<Long> idList = new ArrayList<>(refList.size());
                    refList.forEach(ref -> idList.add(ref.getRoleId()));

                    result = sysRoleService.querySysRole(bean, idList, pager);
                }
            }
            return ResultBean.success("操作成功").setListPager(result).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/queryAllRoleRefUser")
    @ApiOperation(value = "在所有角色中查询角色及关联的人员信息", notes = "查询角色及关联的人员信息")
    public BaseResultBean queryAllRoleRefUser(@RequestBody SysRoleModel sysRoleModel) {
        try {
            ListPager result = null;
            List<SysRole> roleList = null;
            SysRole sysRole = sysRoleModel.getSysRole();
            Pager pager = sysRoleModel.getPager();
            if (ObjectUtils.isEmpty(sysRole)) {
                sysRole = new SysRole();
            }
            if (ObjectUtils.isEmpty(pager)) {
                pager = new Pager();
            }
            roleList = sysRoleService.findAllByRoleProperty(sysRole);
            //过滤SysRefUserRole
            if (ObjectUtils.isEmpty(roleList)){
                return  ResultBean.success("操作成功");
            }

            List<Long> roleIdList = roleList.stream().map(el -> el.getId()).collect(Collectors.toList());
            result = personnelService.findSysRefUserRoleByRoleIds(roleIdList, pager);
            return ResultBean.success("操作成功").setListPager(result).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @PostMapping(value = "/queryRoleRefUser")
    @ApiOperation(value = "查询当前请求人员的角色及关联的人员信息", notes = "查询角色及关联的人员信息")
    public BaseResultBean queryRoleRefUser(@RequestBody SysRoleModel sysRoleModel) {
        try {
            ListPager result = null;
            SysRole sysRole = sysRoleModel.getSysRole();
            Pager pager = sysRoleModel.getPager();
            if (ObjectUtils.isEmpty(sysRole)){
                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            }
            if (ObjectUtils.isEmpty(pager)){
                pager = new Pager();
            }
            if (StringUtils.isEmpty(sysRole.getName())){
                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            }
            List<SysRefUserRole> refList = personnelService.findSysRefUserRoleByRoleNameAndUserID(sysRole.getName(), getSessionUser().getUser().getId());
            if (refList != null && refList.size() > 0) {
                List<Long> idList = new ArrayList<>(refList.size());
                refList.forEach(ref -> {
                    idList.add(ref.getRoleId());
                });

//                result = sysRoleService.querySysRole(sysRole, idList, pager);
//                List<SysRole> roleList = result.getObjectList();
//                //过滤SysRefUserRole
//
//                List<Long> roleIdList = roleList.stream().map(el -> el.getId()).collect(Collectors.toList());
                result = personnelService.findSysRefUserRoleByRoleIds(idList,pager);
            }
            return ResultBean.success("操作成功").setListPager(result).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
    /**
     * 修改角色状态
     *
     * @param id     角色id
     * @param status 状态
     * @return 处理结果
     */
    @PutMapping(value = "/status")
    @ApiOperation(value = "修改角色状态", notes = "根据ID修改角色状态")
    public ResultBean updateStatus(Long id, int status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "角色id 不能为空");
            ValidatorBean.checkNotZero(status, "角色状态不能为空");

            SysRole role = sysRoleService.get(id);
            ValidatorBean.checkNotNull(role, "不存在的角色信息");
            role.setRoleStatus(status);
            ConvertBean.modelUpdate(role, getSessionUser());
            sysRoleService.update(role);
            sysRoleService.insertSysLogRoleOperate(role, CommonEnumUtil.USER_STATUS.valueOfDescription(status));

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 批量修改系统角色状态
     *
     * @param ids    系统角色ids数组
     * @param status 状态
     * @return 处理结果
     */
    @PutMapping(value = "/batch-status")
    @ApiOperation(value = "批量修改系统角色状态", notes = "根据ID修改系统角色状态")
    public ResultBean updateBatchStatus(Long[] ids, int status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(ids, "角色id 不能为空");
            ValidatorBean.checkNotZero(status, "角色状态不能为空");

            sysRoleService.updateSysRoleStatusByIds(ids, status, AuthUtil.getSessionUser());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 新增角色权限关系
     *
     * @param roleId 角色id
     * @param refIds 关系di
     * @return 处理结果
     */
    @PutMapping(value = "/ref-menu/insert")
    @ApiOperation(value = "新增角色权限关系", notes = "新增角色权限关系")
    public ResultBean insertRefMenu(Long roleId, String[] refIds) {
        try {
            startMultiService();

            // 数据校验
            ValidatorBean.checkNotNull(roleId, "角色roleId 不能为空");

            ConvertBean.modelSafeArrayNumber(refIds, true);

            sysRoleService.refreshSysRoleRef(roleId, StringTool.getArrayLong(refIds), AuthUtil.getSessionUser().getUserName());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @PostMapping(value = "/ref-menu/save")
    @ApiOperation(value = "保存角色权限关系", notes = "? 权限列表没有数据时清理角色的产品")
    public ResultBean saveRefMenuBySoftType(@RequestBody SysRefRoleMenu refRoleMenu) {
        try {
            // 数据校验
            ValidatorBean.beginValid(refRoleMenu)
                    .notNull("roleId", refRoleMenu.getRoleId())
                    .notNull("softType", refRoleMenu.getSoftType());
            if (CollectionUtils.isEmpty(refRoleMenu.getMenuIdList())) {
                throw ImppExceptionBuilder.newInstance().setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode()).setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode()).setErrorDetail("未勾选菜单").build();
            }
            sysRoleService.refreshSysRoleRef(refRoleMenu.getRoleId(), refRoleMenu.getSoftType(), refRoleMenu.getMenuIdList()
                    , AuthUtil.getSessionUser().getUserName());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    /**
     * 新增角色权限关系
     *
     * @param roleId   角色id
     * @param softType 产品类型ID
     * @return 处理结果
     */
    @GetMapping(value = "/ref-menu/{roleId}/{softType}/list")
    @ApiOperation(value = "查询角色权限关系", notes = "根据软件类型查询角色权限关系")
    public ResultBean findRefMenuBySoftType(@PathVariable("roleId") Long roleId, @PathVariable("softType") Integer softType) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(roleId, "角色roleId 不能为空");
            ValidatorBean.checkNotNull(softType, "产品类型 不能为空");

            List<SysRefRoleMenu> sysRefRoleMenuList = sysRoleService.findSysRefRoleMenuByRoleId(roleId, softType);

            sysRefRoleMenuList = sysRoleService.setLanguageSysMenuName(AuthUtil.getSessionUser().getLanguageCode(), sysRefRoleMenuList);


            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(sysRefRoleMenuList);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 批量新增角色权限关系
     *
     * @param roleIds 角色id 集合
     * @param menuIds 功能id
     * @return 处理结果
     */
    @PutMapping(value = "/ref-menu/batch-insert")
    @ApiOperation(value = "批量新增角色权限关系", notes = "批量新增角色权限关系")
    public ResultBean insertBatchRefMenu(String[] roleIds, String[] menuIds) {
        try {
            startMultiService();

            roleIds = ConvertBean.modelSafeArrayNumber(roleIds, true);
            menuIds = ConvertBean.modelSafeArrayNumber(menuIds, true);

            // 数据校验
            ValidatorBean.checkNotNull(roleIds, "角色roleIds 不能为空");

            if (roleIds.length == 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请输入需要更新的角色信息。")
                        .build();
            }
            List<SysRole> roleList = sysRoleService.findSysRoleByInIdAndStatus(StringTool.getArrayLong(roleIds), CommonEnumUtil.DATA_STATUS.DISABLE.getValue());

            if (roleList != null && roleList.size() > 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail("[" + roleList.get(0).getName() + "]角色已禁用，不允许操作")
                        .setErrorSolution("请重新操作")
                        .build();
            }
            sysRoleService.refreshBatchSysRoleRef(StringTool.getArrayLong(roleIds), StringTool.getArrayLong(menuIds),
                    AuthUtil.getSessionUser().getUserName());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/export/role-change")
    @ApiOperation(value = "导出角色操作变更日志", notes = "导出角色操作变更日志")
    public ResultBean exportUserStatusChange(@RequestBody ExportDataModel model, HttpServletResponse response) {
        File file = null;

        try {
            ValidatorBean.checkNotNull("ColName", model.getExportCol());
            ValidatorBean.checkNotNull("startDate", model.getStartDate());
            ValidatorBean.checkNotNull("endDate", model.getEndDate());

            final String ZERO_TIME = " 00:00:00";
            final String LAST_TIME = " 23:59:59";
            StringBuilder startTime = new StringBuilder(model.getStartDate()).append(ZERO_TIME);
            StringBuilder endTime = new StringBuilder(model.getEndDate()).append(LAST_TIME);

            LinkedHashMap<String, String> exportColMap = JSON.parseObject(model.getExportCol(), LinkedHashMap.class);
            List<SysLogRoleOperate> list = sysLogRoleOperateService.findLogsBetweenDatetime(startTime.toString(), endTime.toString());
            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            String fileName = "role-change" + System.currentTimeMillis() + ".xls";
            file = new File(fileName);
            file.createNewFile();
            excelTool.exportData(file, list, SysLogRoleOperate.class
                    , exportColMap);

            response.setContentType("application/force-download"); // 设置强制下载不打开
            response.addHeader("Content-Disposition", "attachment;fileName=" + fileName); // 设置文件名
            response.addHeader("Content-type", FileContentTypeTool.getContentType(StringTool.getStringFileSuffix(fileName, true)));

            // 设置文件名
            try (BufferedInputStream bis = new BufferedInputStream(new DataInputStream(new FileInputStream(file)))) {
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                LOGGER.error("角色操作变更日志导出异常", e);
            }
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            if (file != null) {
                FileUtils.deleteQuietly(file);
            }
        }
    }

    @PostMapping(value = "/export/role-menu-change")
    @ApiOperation(value = "导出角色权限变更日志", notes = "导出角色权限变更日志")
    public ResultBean exportroleMenuChange(@RequestBody ExportDataModel model, HttpServletResponse response) {
        File file = null;

        try {
            ValidatorBean.checkNotNull("ColName", model.getExportCol());
            ValidatorBean.checkNotNull("startDate", model.getStartDate());
            ValidatorBean.checkNotNull("endDate", model.getEndDate());

            final String ZERO_TIME = " 00:00:00";
            final String LAST_TIME = " 23:59:59";
            StringBuilder startTime = new StringBuilder(model.getStartDate()).append(ZERO_TIME);
            StringBuilder endTime = new StringBuilder(model.getEndDate()).append(LAST_TIME);

            LinkedHashMap<String, String> exportColMap = JSON.parseObject(model.getExportCol(), LinkedHashMap.class);
            List<SysLogRoleMenuChange> list = sysLogRoleMenuChangeService.findLogsBetweenDatetime(startTime.toString(), endTime.toString());
            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            String fileName = "role-menu-change" + System.currentTimeMillis() + ".xls";
            file = new File(fileName);
            file.createNewFile();
            excelTool.exportData(file, list, SysLogRoleMenuChange.class
                    , exportColMap);

            response.setContentType("application/force-download"); // 设置强制下载不打开
            response.addHeader("Content-Disposition", "attachment;fileName=" + fileName); // 设置文件名
            response.addHeader("Content-type", FileContentTypeTool.getContentType(StringTool.getStringFileSuffix(fileName, true)));

            // 设置文件名
            try (BufferedInputStream bis = new BufferedInputStream(new DataInputStream(new FileInputStream(file)))) {
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                LOGGER.error("角色权限变更日志导出异常", e);
            }
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            if (file != null) {
                FileUtils.deleteQuietly(file);
            }
        }
    }
}
