package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.api.iservice.busi.ISysPojoVersionDetailService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysPojoVersionPlanService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysPojoVersionService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.platform.bean.SysConfig;
import cn.estsh.i3plus.pojo.platform.bean.SysPojoVersion;
import cn.estsh.i3plus.pojo.platform.bean.SysPojoVersionDetail;
import cn.estsh.i3plus.pojo.platform.bean.SysPojoVersionPlan;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import com.alibaba.fastjson.JSON;
import com.google.common.collect.MapDifference;
import com.google.common.collect.Maps;
import com.rabbitmq.client.Channel;
import net.bull.javamelody.internal.common.LOG;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;
import java.util.*;

import static cn.estsh.i3plus.platform.common.util.PlatformConstWords.QUEUE_IMPP_POJO_VERSION;

/**
 * @Description :
 * 对象版本记录
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-4-26 下午2:32
 * @Modify:
 **/
@ConditionalOnExpression("'${impp.mq.queue.pojo.version:true}' == 'true'")
@Configuration
public class PojoVersionQueueReceiver {
    private static final Logger LOGGER = LoggerFactory.getLogger(PojoVersionQueueReceiver.class);

    @Resource
    private ISysPojoVersionService pojoVersionService;

    @Resource
    private ISysPojoVersionDetailService pojoVersionDetailService;

    @Resource
    private SnowflakeIdMaker snowflakeIdMaker;

    @Resource
    private ISysPojoVersionPlanService sysPojoVersionPlanService;

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    /**
     * QUEUE_SWEB_NOTICE 队列
     *
     * @return 队列
     * @throws Exception
     */
    @Bean
    public Queue getQueueSwebNoticeQueue() {
        return new Queue(QUEUE_IMPP_POJO_VERSION);
    }

    @RabbitListener(queues = QUEUE_IMPP_POJO_VERSION)
    public Boolean processImppMessage(Channel channel, Message message) {
        boolean isNack = false;
        LOGGER.info("PojoVersionQueueReceiver开始消费数据:{}",message);
        try {
            byte[] messageBody = message.getBody();
            if (messageBody != null && messageBody.length > 0) {
                String msg = new String(messageBody);
                if (StringUtils.isNotBlank(msg)) {
                    // 当前操作数据
                    SysPojoVersion afterVersion = JSON.parseObject(msg, SysPojoVersion.class);

                    String redisKey = CommonConstWords.REDIS_PREFIX_POJO_VERSION + ":" + afterVersion.getSoftType() + ":" + afterVersion.getRefClass();
                    // 是否进行版本控制
                    Object redisValue = redisRes.getObject(redisKey);
                    if (Objects.isNull(redisValue)){
                        //去查询数据库
                        List<Long> idList = new ArrayList<>();
                        idList.add(afterVersion.getRefId());
                        List<SysPojoVersionPlan> plans = sysPojoVersionPlanService.findByIdList(idList);
                        if (!Objects.isNull(plans) && !plans.isEmpty()){
                            redisRes.putObject(redisKey,JSON.toJSONString(plans.get(0)),-1);
                            redisValue = plans.get(0);
                        }
                    }
                    LOGGER.info("PojoVersionQueueReceiver--是否进行版本控制的key:{},value:{}",redisKey,redisValue);
                    if (Objects.nonNull(redisValue)) {
                        SysPojoVersionPlan versionPlan = JSON.parseObject(redisValue.toString(), SysPojoVersionPlan.class);
                        Map<String, Object> afterMap = JsonUtilTool.decode(afterVersion.getBean(), Map.class);
                        SysPojoVersionDetail afterVersionDetail = null;
                        Map<String, String> mapPropDesc = afterVersion.getPojoPropDesc();

                        // 已存在数据
                        SysPojoVersion beforeVersion = null;
                        SysPojoVersionDetail beforeVersionDetail = null;
                        Map<String, Object> beforeMap = new HashMap<>();

                        Integer versionNo;                                                                      // 对象版本
                        CommonEnumUtil.DAO_OPERATE_TYPE operateType = CommonEnumUtil.DAO_OPERATE_TYPE.INSERT;   // 操作类型
                        Map<String, MapDifference.ValueDifference<Object>> mapDiff;                           // 版本对比
                        String userName = StringUtils.isNotBlank(afterVersion.getModifyUser()) ? afterVersion.getModifyUser() : afterVersion.getCreateUser();

                        if (!"insert".equals(afterVersion.getVersionMethodName())) {  // 不是新增数据的时候则需要查询历史记录
                            // 已存在数据封装
                            afterVersion.setOrderByParam("createDatetime");
                            afterVersion.setAscOrDesc(CommonEnumUtil.ASC_OR_DESC.DESC.getValue());
                            beforeVersion = pojoVersionService.getPojoVersion(afterVersion);
                            if (Objects.nonNull(beforeVersion)) {
                                beforeVersionDetail = pojoVersionDetailService.get(beforeVersion.getPojoDetailId());
                                if (Objects.nonNull(beforeVersionDetail)) {
                                    beforeMap.putAll(JsonUtilTool.decode(beforeVersionDetail.getPojoAfter(), Map.class));
                                }
                            }
                            operateType = CommonEnumUtil.DAO_OPERATE_TYPE.UPDATE;
                        }

                        // 不保存字段 数据剔除
                        for (String key : CommonConstWords.POJO_VERSION_SAVE_ATTR_REMOVE) {
                            afterMap.remove(key);
                        }

                        mapDiff = Maps.difference(beforeMap, afterMap).entriesDiffering();
//                        // 忽略版本控制属性
//                        if(differing != null ){
//                            for (String key : CommonConstWords.POJO_VERSION_ATTR_INGNORE) {
//                                if(differing.containsKey(key)){
//                                    differing.remove(key);
//                                }
//                            }
//                            pojoCompare = differing.toString();
//                        }
                        String pojoCompare = mapDiff == null ? null : mapDiff.toString();
                        if ("insert".equals(afterVersion.getVersionMethodName())) { //
                            pojoCompare = JSON.toJSONString(afterMap);
                        }

                        versionNo = Objects.nonNull(beforeVersion) ? beforeVersion.getVersionNo() : 0;
                        afterVersion.setVersionNo(++versionNo);
                        afterVersion.setId(snowflakeIdMaker.nextId());
                        afterVersion.setPlanId(versionPlan.getId());


                        afterVersionDetail = new SysPojoVersionDetail();
                        afterVersionDetail.setId(snowflakeIdMaker.nextId());
                        afterVersionDetail.setPojoVersionId(afterVersion.getId());

                        afterVersionDetail.setPojoBefore(JSON.toJSONString(beforeMap));
                        afterVersionDetail.setPojoAfter(JSON.toJSONString(afterMap));
                        afterVersionDetail.setPojoCompare(pojoCompare);

                        afterVersionDetail.setPojoBeforeTxt(getMapToStr(beforeMap, mapPropDesc));
                        afterVersionDetail.setPojoAfterTxt(getMapToStr(afterMap, mapPropDesc));
                        afterVersionDetail.setPojoCompareTxt(getMapDiffToStr(mapDiff, mapPropDesc));

                        ConvertBean.serviceModelInitialize(afterVersionDetail, userName);
                        pojoVersionDetailService.insert(afterVersionDetail);

                        afterVersion.setOperateType(operateType.getValue());
                        afterVersion.setVersionNo(versionNo);
                        afterVersion.setPojoCompare(pojoCompare);
                        afterVersion.setPojoCompareTxt(getMapDiffToStr(mapDiff, mapPropDesc));
                        afterVersion.setPojoDetailId(afterVersionDetail.getId());
                        ConvertBean.serviceModelInitialize(afterVersion, userName);
                        pojoVersionService.insert(afterVersion);
                        LOGGER.info("PojoVersionQueueReceiver:保存的pojoVersion:{}",afterVersion);
                    }
                }
            }
        } catch (Exception e) {
            LOGGER.error("PojoVersionQueueReceiver 消息处理失败{}", new String(message.getBody()), e);
            isNack = true;
        } finally {
            try {
                if (isNack) {
                    channel.basicNack(message.getMessageProperties().getDeliveryTag(), false, true);
                } else {
                    channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                }
            } catch (Exception e) {
                LOGGER.error("PojoVersionQueueReceiver Error Message :{}", e.getMessage());
            }
        }
        return !isNack;
    }

    /**
     * Map数据转文本
     *
     * @param mapData
     * @param mapPropDesc
     * @return
     */
    private String getMapToStr(Map<String, Object> mapData, Map<String, String> mapPropDesc) {
        if (mapData == null) {
            mapData = new HashMap<>();
        }
        if (mapPropDesc == null) {
            mapData = new HashMap<>();
        }

        StringBuilder stringBuffer = new StringBuilder();
        String key;
        for (Map.Entry<String, Object> entry : mapData.entrySet()) {
            key = mapPropDesc.get(entry.getKey());
            if (key == null) {
                key = entry.getKey();
            }
            stringBuffer.append(key).append(":").append(entry.getValue()).append(",");
        }
        return stringBuffer.toString();
    }

    /**
     * MapDifference数据转文本
     *
     * @param mapData
     * @param mapPropDesc
     * @return
     */
    private String getMapDiffToStr(Map<String, MapDifference.ValueDifference<Object>> mapData, Map<String, String> mapPropDesc) {
        if (mapData == null) {
            mapData = new HashMap<>();
        }
        if (mapPropDesc == null) {
            mapData = new HashMap<>();
        }

        StringBuilder stringBuffer = new StringBuilder();
        String key;
        MapDifference.ValueDifference<Object> valDiff;
        for (Map.Entry<String, MapDifference.ValueDifference<Object>> entry : mapData.entrySet()) {
            valDiff = entry.getValue();
            if (valDiff == null) {
                continue;
            }
            key = mapPropDesc.get(entry.getKey());
            if (key == null) {
                key = entry.getKey();
            }
            stringBuffer.append("修改值 ").append(key)
                    .append("，旧值为 ").append(valDiff.leftValue())
                    .append("，新值为 ").append(valDiff.rightValue())
                    .append("。");
        }
        return stringBuffer.toString();
    }

    public static void main(String[] args) {
        Map<String, String> orginMap = new HashMap<>();
        orginMap.put("name", "wyh");
        orginMap.put("age", "18");
        orginMap.put("sex", "male");
        Map<String, String> newMap = new HashMap<>();
        newMap.put("name", "hyw");
        newMap.put("age", "81");
        MapDifference mapDiff = Maps.difference(orginMap, newMap);
        System.out.println(mapDiff.entriesDiffering());
        System.out.println(mapDiff.entriesInCommon());
        System.out.println(mapDiff.entriesOnlyOnLeft());
        System.out.println(mapDiff.entriesOnlyOnRight());
        System.out.println(mapDiff.areEqual());

        System.out.println(ExcelTool.getColName(SysConfig.class));
    }

}