package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysPojoVersionDetailService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysPojoVersionService;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.jpa.dao.BaseRepository;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.platform.bean.SysPojoVersion;
import cn.estsh.i3plus.pojo.platform.bean.SysPojoVersionPlan;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserRole;
import cn.estsh.i3plus.pojo.platform.model.SysVersionExportModel;
import cn.estsh.i3plus.pojo.platform.model.SysVersionModel;
import cn.estsh.i3plus.pojo.platform.repository.SysPojoVersionPlanRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysPojoVersionRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysRefUserRoleRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.base.service.CrudService;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-4-21 下午2:28
 * @Modify:
 **/
@Service
public class SysPojoVersionService extends CrudService<SysPojoVersion> implements ISysPojoVersionService {

    @Autowired
    private SysPojoVersionPlanRepository pojoVersionPlanRepository;

    @Autowired
    private SysPojoVersionRepository pojoVersionRDao;

    @Autowired
    private ISysPojoVersionDetailService pojoVersionDetailService;

    @Autowired
    private SysRefUserRoleRepository refUserRoleRepository;

    @Override
    public BaseRepository<SysPojoVersion, Long> getRepository() {
        return pojoVersionRDao;
    }


    @Override
    public SysPojoVersion getPojoVersion(SysPojoVersion bean) {
        return pojoVersionRDao.getByProperty(DdlPackBean.getDdlPackBean(bean));
    }

    @Override
    public List<SysPojoVersion> findPojoVersionByUserBetweenTime(SysVersionExportModel model) {
        String endDate = model.getEndDate();
        String startDate = model.getStartDate();
        String userName = model.getUserName();
        Long roleId = model.getRoleId();
        DdlPackBean packBean = DdlPackBean.getDdlPackBean();
        DdlPackBean packBeanPlan = DdlPackBean.getDdlPackBean();
        if (Strings.isNotBlank(userName)){
            DdlPreparedPack.getStringEqualPack(userName, "modifyUser", packBean);
        }
        if (!Objects.isNull(roleId) && Strings.isBlank(userName)){
            DdlPackBean roleRefPackBean = DdlPackBean.getDdlPackBean();
            DdlPreparedPack.getNumEqualPack(roleId,"roleId",roleRefPackBean);
            List<SysRefUserRole> refUserRoles = refUserRoleRepository.findByHqlWhere(roleRefPackBean);
            List<String> userNameList = refUserRoles.stream().map(SysRefUserRole::getUserNameRdd).collect(Collectors.toList());
            DdlPreparedPack.getInPackList(userNameList.stream().distinct().collect(Collectors.toList()), "modifyUser",packBean);
        }
//        DdlPackBean planPackBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.timeBuilder(startDate, endDate, "modifyDatetime", true, false, packBean);
        List<SysPojoVersion> pojoVersions = pojoVersionRDao.findByHqlWhere(packBean);

        DdlPreparedPack.getNumEqualPack(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(),"isValid",packBeanPlan);
        DdlPreparedPack.getNumEqualPack(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),"isDeleted",packBeanPlan);
        List<SysPojoVersionPlan> pojoVersionPlans = pojoVersionPlanRepository.findByHqlWhere(packBeanPlan);


        pojoVersions.stream().parallel().forEach(d -> {
            Optional<SysPojoVersionPlan> plan = pojoVersionPlans.stream().filter(p -> d.getPlanId().equals(p.getId())).findFirst();
            plan.ifPresent(c -> d.setPojoClassTitle(c.getRefClassTitle()));
        });

        return pojoVersions;
    }

    @Override
    public ListPager<SysPojoVersion> findPojoVersionByModel(SysVersionModel model, Pager pager) {
//        String organizeCode = AuthUtil.getOrganizeCode();
        Long roleId = model.getRoleId();
        List<SysRefUserRole> sysRefUserRoleList = new ArrayList<>();
        if (!Objects.isNull(roleId)){
            //查询角色查询出所有的拥有该角色的人员名称
            DdlPackBean rolePackBean = DdlPackBean.getDdlPackBean();
            DdlPreparedPack.getNumEqualPack(roleId,"roleId",rolePackBean);
            sysRefUserRoleList.addAll(refUserRoleRepository.findByHqlWhere(rolePackBean));
        }

        DdlPackBean versionPackBean = DdlPackBean.getDdlPackBean();
        Long planId = model.getPlanId();
        String pojoCompare = model.getPojoCompare();
        Long refId = model.getRefId();
        String modifyDateTimeStart = model.getModifyDateTimeStart();
        String modifyDateTimeEnd = model.getModifyDateTimeEnd();
        if (!Objects.isNull(planId)){
            DdlPreparedPack.getNumEqualPack(planId,"planId",versionPackBean);
        }
        if (Strings.isNotBlank(pojoCompare)){
            DdlPreparedPack.getStringRightLikerPack(pojoCompare,"pojoCompare",versionPackBean);
        }

        if (!Objects.isNull(refId)){
            DdlPreparedPack.getNumEqualPack(refId,"refId",versionPackBean);
        }

        if (Strings.isNotBlank(modifyDateTimeStart) && Strings.isNotBlank(modifyDateTimeEnd)){
            DdlPreparedPack.timeBuilder(modifyDateTimeStart,modifyDateTimeEnd,"createDatetime",versionPackBean,true);
        }
        if (sysRefUserRoleList.size() > 0){
            List<String> userNameList = sysRefUserRoleList.stream().map(SysRefUserRole::getUserNameRdd).distinct().collect(Collectors.toList());
            DdlPreparedPack.getInPackList(userNameList,"createUser",versionPackBean);
        }
        int count = pojoVersionRDao.findByHqlWhereCount(versionPackBean);
        pager = PagerHelper.getPager(pager, count);
        List<SysPojoVersion> sysPojoVersionList = pojoVersionRDao.findByHqlWherePage(versionPackBean, pager);
        return new ListPager<>(sysPojoVersionList,pager);
    }

    @Override
    public SysPojoVersion packGetBean(SysPojoVersion bean) {
        if (Objects.nonNull(bean)) {
            bean.setVersionDetail(pojoVersionDetailService.get(bean.getPojoDetailId()));
        }
        return bean;
    }

    @Override
    public DdlPackBean getFindPagerPackBean(SysPojoVersion bean) {
        return CoreHqlPack.packHqlSysPojoVersion(bean);
    }
}
